@@ -1,6 +1,6 @@
 package TPath::Test::Node::Complement;
 {
-  $TPath::Test::Node::Complement::VERSION = '1.001';
+  $TPath::Test::Node::Complement::VERSION = '1.002';
 }
 
 # ABSTRACT: L<TPath::Test::Node> implementing matching; e.g., C<//^~foo~>, C<//^foo>, and C<//^@foo>
@@ -16,8 +16,9 @@ has nt => ( is => 'ro', isa => 'TPath::Test::Node', required => 1 );
 
 # required by TPath::Test::Node
 sub passes {
-    my ( $self, $ctx ) = @_;
-    return $self->nt->passes($ctx) ? undef : 1;
+
+    # my ( $self, $ctx ) = @_;
+    return $_[0]->nt->passes( $_[1] ) ? undef : 1;
 }
 
 __PACKAGE__->meta->make_immutable;
@@ -34,7 +35,7 @@ TPath::Test::Node::Complement - L<TPath::Test::Node> implementing matching; e.g.
 
 =head1 VERSION
 
-version 1.001
+version 1.002
 
 =head1 ATTRIBUTES
 
